﻿
using System;

namespace ZweiDimensionalerRegressator
{

	public class ModellParameter
	{
		public uint NumberOBetasphericDimensions;
		public uint Vec2Length;
		public double[] Gammas;
		public double[,] Alphas;
		public double[,] Betas;
		public bool IstGradient;
		public double LetzteFeherbewertung;
		public double[] GammaVariabel;
		public double[,] AlphaIsVariable;
		private bool FreiheitsgradeAktuell;
		private int FreieParameter;
		
		public ModellParameter(uint _NumberOBetasphericDimensions, uint _Vec2Length)
		{
			if ( (_NumberOBetasphericDimensions == 0) )
				throw new ArgumentException("Mindestens ein Gamma erforderlich");
			if (_Vec2Length == 0)
				throw new ArgumentException("Vec2 muss mindestens ein Feld umfassen");
			
			NumberOBetasphericDimensions = _NumberOBetasphericDimensions;
			Vec2Length = _Vec2Length;
			Gammas = new double[NumberOBetasphericDimensions];
			GammaVariabel = new double[NumberOBetasphericDimensions];
			Alphas = new double[NumberOBetasphericDimensions,Vec2Length];
			Betas = new double[NumberOBetasphericDimensions,Vec2Length];
			AlphaIsVariable = new double[NumberOBetasphericDimensions,Vec2Length];
			IstGradient = false;
			for (int i = 0 ; i < NumberOBetasphericDimensions; i++)
			{
				for (int j = 0; j < Vec2Length; j++)
				{
					AlphaIsVariable[i,j] = 1;
				}
				GammaVariabel[i] = 1;
			}
			FreiheitsgradeAktuell = false;
			FreieParameter = 0;
			
		}
		
		public void NormiereAlphaSummenAufEins()
		{
			int i;
			int j;
			double temp;
			for (i = 0; i < Vec2Length; i++)
			{
				temp = 0;
				for (j = 0; j < NumberOBetasphericDimensions; j++)
				{
					if (Alphas[j,i] < 0)
						Alphas[j,i] = 0;
					temp = temp + Alphas[j,i];
				}

				if (temp != 0)
				{
					for (j = 0; j < NumberOBetasphericDimensions; j++)
						Alphas[j,i] = Alphas[j,i] / temp;
				}
				
				
			}
			FreiheitsgradeAktuell = false;
		}
		
		public void NormiereBetasSummenAufEins()
		{
			int i;
			int j;
			double temp;
			for (i = 0; i < Vec2Length; i++)
			{
				temp = 0;
				for (j = 0; j < NumberOBetasphericDimensions; j++)
				{
					if (Betas[j,i] < 0)
						Betas[j,i] = 0;
					temp = temp +Betas[j,i];
				}

				if (temp != 0)
				{
					for (j = 0; j < NumberOBetasphericDimensions; j++)
						Betas[j,i] = Betas[j,i] / temp;
				}
				
				
			}
			FreiheitsgradeAktuell = false;
		}
		
		public void BerechneAlphasAusBetas()
		{
			int i;
			int j;
			for (i = 0; i < Vec2Length; i++)
			{
				for (j = 0; j < NumberOBetasphericDimensions; j++)
				{
					Alphas[j,i] = Betas[j,i] / Gammas[j];
				}
			}
			FreiheitsgradeAktuell = false;
			NormiereAlphaSummenAufEins();
		}
		

		
		public ModellParameter ErstelleWertKopie(ref ModellParameter _Modellparameter, ref ModellParameter WertKopie)
		{
			//ModellParameter WertKopie = new ModellParameter(_Modellparameter.NumberOBetasphericDimensions,_Modellparameter.Vec2Length);
			int j;
			for (int i = 0; i < _Modellparameter.NumberOBetasphericDimensions; i++)
			{
				for (j = 0; j < _Modellparameter.Vec2Length; j++)
				{
					WertKopie.Betas[i,j] = _Modellparameter.Betas[i,j];
					WertKopie.Alphas[i,j] = _Modellparameter.Alphas[i,j];
					WertKopie.AlphaIsVariable[i,j] = _Modellparameter.AlphaIsVariable[i,j];
				}
				WertKopie.Gammas[i] = _Modellparameter.Gammas[i];
				WertKopie.GammaVariabel[i] = _Modellparameter.GammaVariabel[i];
			}
			return WertKopie;
		}
		
		public double BerechneMaximaleBetragsAbweichungZweierModellParameter(ModellParameter _MP1, ModellParameter _MP2)
		{

			
			
			// MBA = Maximale Betrags Abweichung
			double MBA = 0;
			int j;
			for (int i = 0; i < _MP1.NumberOBetasphericDimensions; i++)
			{
				for (j = 0; j < _MP1.Vec2Length; j++)
				{
					MBA = Math.Max(MBA, Math.Abs(_MP1.Betas[i,j] - _MP2.Betas[i,j]));
				}
				MBA = Math.Max(MBA, Math.Abs(_MP1.Gammas[i] - _MP2.Gammas[i]));
			}
			return MBA;
		}
		
		public int GebeAnzahlFreierParameter ()
		{
			// Wenn schon berechnet, dann einfach zurück geben
			if (FreiheitsgradeAktuell == true)
				return FreieParameter;
			
			
			FreieParameter = 0;
			

			
			for (int i = 0; i < NumberOBetasphericDimensions; i++)
			{
				if (i > 0)
				{
					for (int j = 0; j < Vec2Length; j++)
					{
						if (AlphaIsVariable[i,j] != 0)
							FreieParameter++;
					}
				}
				if (GammaVariabel[i] != 0)
					FreieParameter++;
			}
			FreiheitsgradeAktuell = true;
			return FreieParameter;
		}
		
		public double Maximum_dDeltaMat_dpartial()
		{
			int i = 0;
			int j = 0;
			double MaxDev = 0;
			for (i = 0; i < NumberOBetasphericDimensions; i++)
			{
				for (j = 0; j < Vec2Length; j++)
				{
					if (AlphaIsVariable[i,j] != 0)
					{
						if (MaxDev < Math.Abs(Betas[i,j]))
						{
							MaxDev = Math.Abs(Betas[i,j]);
						}
					}
				}
				if (GammaVariabel[i] != 0)
				{
					if (MaxDev < Math.Abs(Gammas[i]))
					{
						MaxDev = Math.Abs(Gammas[i]);
					}
				}
			}
			return MaxDev;
		}

		
	}
}
